﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Polly.Contrib.Simmy.Fault;
using SimmyFault.Model;

namespace SimmyFault.Controllers
{
    [Produces("application/json")]
    [Route("api/[controller]")]
    public class BreweryNoHttpClientFactoryController : Controller
    {
        private readonly AsyncInjectOutcomePolicy<HttpResponseMessage> _faultPolicy;
        private readonly HttpClient _httpClient;
        public BreweryNoHttpClientFactoryController(AsyncInjectOutcomePolicy<HttpResponseMessage> faultPolicy)
        {
            _httpClient = new HttpClient();
            _httpClient.BaseAddress = new Uri("https://api.openbrewerydb.org/breweries/");
            _httpClient.DefaultRequestHeaders.Add("Accept", "application/json");
            _faultPolicy = faultPolicy;
        }

        public async Task<IActionResult> Get(string state = "Massachusetts", string name = "night shift brewing")
        {
            string requestEndpoint = $"?by_state={state}&by_name={name}";

            var response = await _faultPolicy.ExecuteAsync(() => _httpClient.GetAsync(requestEndpoint));
            if (response.IsSuccessStatusCode)
            {
                var breweries = await response.Content.ReadAsAsync<List<Brewery>>();
                return Ok(breweries);
            }

            return StatusCode((int)response.StatusCode, response.Content.ReadAsStringAsync());
        }
    }
}
