﻿using System;
using System.Net.Http;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Polly.Contrib.Simmy;
using Polly.Contrib.Simmy.Fault;

namespace SimmyFault
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            
            AsyncInjectOutcomePolicy<HttpResponseMessage> faultPolicy = MonkeyPolicy.InjectFaultAsync<HttpResponseMessage>(
                new HttpRequestException("Simmy threw an exception"), 
                injectionRate: .5, //can be set through configuration
                enabled: () => true //can be set through configuration
            );

            // this is for people not using HttpClientFactory
            services.AddSingleton<AsyncInjectOutcomePolicy<HttpResponseMessage>>(faultPolicy);

            // Add the HttpClientFactory
            services.AddHttpClient("OpenBreweryDb", client =>
            {
                client.BaseAddress = new Uri("https://api.openbrewerydb.org/breweries/");
                client.DefaultRequestHeaders.Add("Accept", "application/json");
            }).AddPolicyHandler(faultPolicy);

            services.AddMvc().SetCompatibilityVersion(CompatibilityVersion.Version_2_2);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseMvc();
        }
    }
}
